@@ -1,16 +1,12 @@
 from django.db import connections
 from django.db.models.expressions import Col
 from django.db import models
+from .utils import copy_model, chunks
+import random
 import datetime
 import traceback
 
 
-def chunks(source_list, batch_size):
-    """Yield successive n-sized chunks from source_list."""
-    for i in range(0, len(source_list), batch_size):
-        yield source_list[i:i+batch_size]
-
-
 class ShardException(Exception):
     pass
 
@@ -24,29 +20,19 @@ class ShardManager(models.Manager):
         Usage: Model.objects.shard(1).all()
         """
         meta = getattr(self.model, '_meta')
-        meta._db = db
-        meta.db_table = '%s_%s_%s' % (
+        db_table = '%s_%s_%s' % (
             str(meta.app_label),
             str(self.model.__name__.lower()), table_suffix)
-        # Without clearing the concrete cached fields, we cannot switch between shards in a single session.
-        for f in meta.concrete_fields:
-            f.cached_col = Col(meta.db_table, f)
-        return super(ShardManager, self).using(self.db)
-
-    def default(self):
-        """
-        Use the original table.
-        Usage: Model.objects.default().all()
-        """
-        meta = getattr(self.model, '_meta')
-        meta.db_table = '%s_%s' % (
-            str(meta.app_label),
-            str(self.model.__name__.lower()))
-        meta._db = 'default'
-        # Without clearing the concrete cached fields, we cannot switch between shards in a single session.
-        for f in meta.concrete_fields:
-            f.cached_col = Col(meta.db_table, f)
-        return super(ShardManager, self).using(self.db)
+        model_name = 'ShardedModel-%s' % random.randint(999999999, 9999999999999999)
+        self.model = copy_model(
+            model_name,
+            self.model,
+            options={'db_table': db_table, 'auto_created': False}
+        )
+        new_meta = getattr(self.model, '_meta')
+        for f in new_meta.concrete_fields:
+            f.cached_col = Col(db_table, f)
+        return super(ShardManager, self).using(db)
 
     def create(self, table_suffix, db='default', **kwargs):
         meta = getattr(self.model, '_meta')
@@ -54,6 +40,7 @@ class ShardManager(models.Manager):
             str(meta.app_label),
             str(self.model.__name__.lower()), table_suffix)
 
+        delete_keys = []
         for k, v in kwargs.items():
             if isinstance(v, datetime.datetime):
                 kwargs[k] = v.strftime('%Y-%m-%d %H:%M:%S')
@@ -66,6 +53,11 @@ class ShardManager(models.Manager):
                     kwargs[k] = 1
                 else:
                     kwargs[k] = 0
+            elif v is None:
+                delete_keys.append(k)
+
+        for delete in delete_keys:
+            del kwargs[delete]
 
         placeholders = ', '.join(['%s'] * len(kwargs.values()))
         columns = ', '.join(kwargs.keys())
@@ -91,6 +83,7 @@ class ShardManager(models.Manager):
             str(self.model.__name__.lower()), table_suffix)
 
         for dict_fields in list_of_dicts:
+            delete_keys = []
             for k, v in dict_fields.items():
                 if isinstance(v, datetime.datetime):
                     dict_fields[k] = v.strftime('%Y-%m-%d %H:%M:%S')
@@ -103,6 +96,10 @@ class ShardManager(models.Manager):
                         dict_fields[k] = 1
                     else:
                         dict_fields[k] = 0
+                elif v is None:
+                    delete_keys.append(k)
+            for delete in delete_keys:
+                del dict_fields[delete]
 
         placeholders = ', '.join(['%s'] * len(list_of_dicts[0]))
         columns = ', '.join(list_of_dicts[0].keys())
@@ -156,17 +153,8 @@ class ShardManager(models.Manager):
 
 
 class ShardedModel(models.Model):
-    def save(self, *args, **kwargs):
-        if len(args) > 0:
-            shard = args[0]
-            db_table = '%s_%s_%s' % (self._meta.app_label, str(self._meta.model.__name__).lower(), shard)
-            self._meta.db_table = db_table
-            for f in self._meta.concrete_fields:
-                f.cached_col = Col(self._meta.db_table, f)
-            return super(ShardedModel, self).save(**kwargs)
-        else:
-            raise ShardException('No shard/table suffix specified in save.')
+
+    objects = ShardManager()
 
     class Meta:
         abstract = True
-
